"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lostAlign = void 0;
const lg_new_block_1 = require("./core/lg-new-block");
const lostAlign = (css, settings) => {
    css.walkDecls('lost-align', function alignDirectionFunction(decl) {
        let declArr = [];
        let flexbox = settings.flexbox;
        declArr = decl.value.split(' ');
        const alignDirection = declArr[0];
        if (declArr.indexOf('flex') !== -1) {
            flexbox = 'flex';
        }
        if (flexbox !== 'flex') {
            if (alignDirection === 'reset') {
                decl.cloneBefore({
                    prop: 'position',
                    value: 'static',
                });
                (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['static', 'auto', 'auto', 'auto', 'auto', 'translate(0, 0)']);
            }
            else {
                decl.cloneBefore({
                    prop: 'position',
                    value: 'relative',
                });
                if (alignDirection === 'horizontal') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', 'auto', 'auto', 'auto', '50%', 'translate(-50%, 0)']);
                }
                else if (alignDirection === 'vertical') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '50%', 'auto', 'auto', 'auto', 'translate(0, -50%)']);
                }
                else if (alignDirection === 'top-left') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '0', 'auto', 'auto', '0', 'translate(0, 0)']);
                }
                else if (alignDirection === 'top-center' ||
                    alignDirection === 'top') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '0', 'auto', 'auto', '50%', 'translate(-50%, 0)']);
                }
                else if (alignDirection === 'top-right') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '0', '0', 'auto', 'auto', 'translate(0, 0)']);
                }
                else if (alignDirection === 'middle-left' ||
                    alignDirection === 'left') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '50%', 'auto', 'auto', '0', 'translate(0, -50%)']);
                }
                else if (alignDirection === 'middle-center' ||
                    alignDirection === 'center') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '50%', 'auto', 'auto', '50%', 'translate(-50%, -50%)']);
                }
                else if (alignDirection === 'middle-right' ||
                    alignDirection === 'right') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', '50%', '0', 'auto', 'auto', 'translate(0, -50%)']);
                }
                else if (alignDirection === 'bottom-left') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', 'auto', 'auto', '0', '0', 'translate(0, 0)']);
                }
                else if (alignDirection === 'bottom-center' ||
                    alignDirection === 'bottom') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', 'auto', 'auto', '0', '50%', 'translate(-50%, 0)']);
                }
                else if (alignDirection === 'bottom-right') {
                    (0, lg_new_block_1.newBlock)(decl, ' > *', ['position', 'top', 'right', 'bottom', 'left', 'transform'], ['absolute', 'auto', '0', '0', 'auto', 'translate(0, 0)']);
                }
                else {
                    throw decl.error(`lost-align: direction '${alignDirection}' is unknown.`);
                }
            }
        }
        else if (alignDirection === 'reset') {
            decl.cloneBefore({
                prop: 'display',
                value: 'initial',
            });
            (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['inherit', 'inherit']);
        }
        else {
            decl.cloneBefore({
                prop: 'display',
                value: 'flex',
            });
            if (alignDirection === 'horizontal') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['center', 'inherit']);
            }
            else if (alignDirection === 'vertical') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['inherit', 'center']);
            }
            else if (alignDirection === 'top-left') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-start', 'flex-start']);
            }
            else if (alignDirection === 'top-center' || alignDirection === 'top') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['center', 'flex-start']);
            }
            else if (alignDirection === 'top-right') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-end', 'flex-start']);
            }
            else if (alignDirection === 'middle-left' ||
                alignDirection === 'left') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-start', 'center']);
            }
            else if (alignDirection === 'middle-center' ||
                alignDirection === 'center') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['center', 'center']);
            }
            else if (alignDirection === 'middle-right' ||
                alignDirection === 'right') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-end', 'center']);
            }
            else if (alignDirection === 'bottom-left') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-start', 'flex-end']);
            }
            else if (alignDirection === 'bottom-center' ||
                alignDirection === 'bottom') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['center', 'flex-end']);
            }
            else if (alignDirection === 'bottom-right') {
                (0, lg_new_block_1.newBlock)(decl, '', ['justify-content', 'align-items'], ['flex-end', 'flex-end']);
            }
            else {
                throw decl.error(`lost-align: direction '${alignDirection}' is unknown.`);
            }
        }
        decl.remove();
    });
};
exports.lostAlign = lostAlign;
