"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lostCenter = void 0;
const lg_new_block_1 = require("./core/lg-new-block");
const lg_logic_1 = require("./core/lg-logic");
const lg_utilities_1 = require("./core/lg-utilities");
const lostCenter = (css, settings, result) => {
    css.walkDecls('lost-center', function lostCenterFunction(decl) {
        let declArr = [];
        let lostCenterPadding;
        let lostCenterMaxWidth;
        let lostCenterFlexbox = settings.flexbox;
        let lostUnit = settings.gridUnit;
        let lostColumnRounder = settings.rounder;
        const lostColumnGutter = '0';
        const validUnits = ['%', 'vw'];
        const isFractionValue = (value) => {
            const lostFractionPattern = /^\d+\/\d+$/;
            return lostFractionPattern.test(value);
        };
        const sanitizedDecl = lg_utilities_1.lgUtils.glueFractionMembers(decl.value);
        declArr = sanitizedDecl.split(' ');
        if (declArr[1] !== undefined && declArr[1].search(/^\d/) !== -1) {
            lostCenterPadding = declArr[1];
        }
        if (lostCenterFlexbox !== 'flex') {
            lostCenterFlexbox = settings.flexbox;
        }
        if (declArr.indexOf('flex') !== -1) {
            lostCenterFlexbox = 'flex';
        }
        if (declArr.indexOf('no-flex') !== -1) {
            lostCenterFlexbox = 'no-flex';
        }
        lostCenterFlexbox = lg_logic_1.lgLogic.parseLostProperty(decl.parent.nodes, 'lost-center-flexbox', lostCenterFlexbox);
        lostCenterPadding = lg_logic_1.lgLogic.parseLostProperty(decl.parent.nodes, 'lost-center-padding', lostCenterPadding);
        lostColumnRounder = lg_logic_1.lgLogic.parseLostProperty(decl.parent.nodes, 'lost-column-rounder', lostColumnRounder);
        lostUnit = lg_logic_1.lgLogic.parseLostProperty(decl.parent.nodes, 'lost-unit', lostUnit);
        if (!lg_logic_1.lgLogic.validateUnit(lostUnit, validUnits)) {
            decl.warn(result, `${lostUnit} is not a valid unit for lost-center`);
            lostUnit = settings.gridUnit;
        }
        if (declArr[0] !== undefined && isFractionValue(declArr[0])) {
            lostCenterMaxWidth = lg_logic_1.lgLogic.calcValue(declArr[0], lostColumnGutter, lostColumnRounder, lostUnit);
        }
        else {
            lostCenterMaxWidth = declArr[0];
        }
        if (lostCenterFlexbox === 'no-flex') {
            (0, lg_new_block_1.newBlock)(decl, ':after', ['content', 'display', 'clear'], ["''", 'table', 'both']);
            (0, lg_new_block_1.newBlock)(decl, ':before', ['content', 'display'], ["''", 'table']);
        }
        else {
            decl.cloneBefore({
                prop: 'display',
                value: 'flex',
            });
            decl.cloneBefore({
                prop: 'flex-flow',
                value: 'row wrap',
            });
        }
        decl.cloneBefore({
            prop: 'max-width',
            value: lostCenterMaxWidth,
        });
        decl.cloneBefore({
            prop: 'margin-left',
            value: 'auto',
        });
        decl.cloneBefore({
            prop: 'margin-right',
            value: 'auto',
        });
        if (lostCenterPadding !== undefined) {
            decl.cloneBefore({
                prop: 'padding-left',
                value: lostCenterPadding,
            });
            decl.cloneBefore({
                prop: 'padding-right',
                value: lostCenterPadding,
            });
        }
        decl.remove();
    });
};
exports.lostCenter = lostCenter;
