"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lostMasonryColumn = void 0;
const lg_utilities_1 = require("./core/lg-utilities");
const lostMasonryColumn = (css, settings) => {
    css.walkDecls('lost-masonry-column', function lostMasonryColumnFunction(decl) {
        let declArr = [];
        let lostMasonryColumnRounder = settings.rounder;
        let lostMasonryColumnFlexbox = settings.flexbox;
        let lostMasonryColumnGutter = settings.gutter;
        let lostMasonryColumnGutterUnit;
        function cloneAllBefore(props) {
            Object.keys(props).forEach(function traverseProps(prop) {
                decl.cloneBefore({
                    prop: prop,
                    value: props[prop],
                });
            });
        }
        const sanitizedDecl = lg_utilities_1.lgUtils.glueFractionMembers(decl.value);
        declArr = sanitizedDecl.split(' ');
        const lostMasonryColumn = declArr[0];
        if (declArr[1] !== undefined && declArr[1].search(/^\d/) !== -1) {
            lostMasonryColumnGutter = declArr[1];
        }
        if (declArr.indexOf('flex') !== -1) {
            lostMasonryColumnFlexbox = 'flex';
        }
        if (declArr.indexOf('no-flex') !== -1) {
            lostMasonryColumnFlexbox = 'no-flex';
        }
        decl.parent.nodes.forEach(function lostMasonryColumnRounderFunction(declaration) {
            if (declaration.prop === 'lost-masonry-column-rounder') {
                lostMasonryColumnRounder = declaration.value;
                declaration.remove();
            }
        });
        decl.parent.nodes.forEach(function lostMasonryColumnGutterFunction(declaration) {
            if (declaration.prop === 'lost-masonry-column-gutter') {
                lostMasonryColumnGutter = declaration.value;
                declaration.remove();
            }
        });
        decl.parent.nodes.forEach(function lostMasonryColumnFlexboxFunction(declaration) {
            if (declaration.prop === 'lost-masonry-column-flexbox') {
                if (declaration.value === 'flex') {
                    lostMasonryColumnFlexbox = 'flex';
                }
                declaration.remove();
            }
        });
        if (lostMasonryColumnGutter !== '0') {
            lostMasonryColumnGutterUnit = lostMasonryColumnGutter
                .match(/\D/g)
                .join('');
        }
        else {
            lostMasonryColumnGutterUnit = '';
        }
        if (lostMasonryColumnFlexbox === 'flex') {
            decl.cloneBefore({
                prop: 'flex',
                value: '0 0 auto',
            });
        }
        else {
            decl.cloneBefore({
                prop: 'float',
                value: 'left',
            });
        }
        if (lostMasonryColumnGutter !== '0') {
            cloneAllBefore({
                width: 'calc(' +
                    lostMasonryColumnRounder +
                    '% * ' +
                    lostMasonryColumn +
                    ' - ' +
                    lostMasonryColumnGutter +
                    ')',
                'margin-left': parseInt(lostMasonryColumnGutter, 10) / 2 +
                    lostMasonryColumnGutterUnit,
                'margin-right': parseInt(lostMasonryColumnGutter, 10) / 2 +
                    lostMasonryColumnGutterUnit,
            });
        }
        else {
            cloneAllBefore({
                width: 'calc(' +
                    lostMasonryColumnRounder +
                    '% * ' +
                    lostMasonryColumn +
                    ')',
                'margin-left': parseInt(lostMasonryColumnGutter, 10) / 2,
                'margin-right': parseInt(lostMasonryColumnGutter, 10) / 2,
            });
        }
        decl.remove();
    });
};
exports.lostMasonryColumn = lostMasonryColumn;
