"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lostWaffle = void 0;
const lg_new_block_1 = require("./core/lg-new-block");
const lg_logic_1 = require("./core/lg-logic");
const lg_utilities_1 = require("./core/lg-utilities");
const lostWaffle = (css, settings) => {
    css.walkDecls('lost-waffle', function lostWaffleDeclFunction(decl) {
        let declArr = [];
        const gridDirection = settings.direction;
        let floatRight;
        let lostWaffleCycle;
        let unit = settings.gridUnit;
        let lostWaffleRounder = settings.rounder;
        let lostWaffleGutter = settings.gutter;
        let lostWaffleFlexbox = settings.flexbox;
        const validUnits = ['%', 'vh', 'vw'];
        function cloneAllBefore(props) {
            Object.keys(props).forEach(function traverseProps(prop) {
                decl.cloneBefore({
                    prop: prop,
                    value: props[prop],
                });
            });
        }
        if (settings.cycle === 'auto') {
            lostWaffleCycle = decl.value.split('/')[1];
        }
        else {
            lostWaffleCycle = settings.cycle;
        }
        const sanitizedDecl = lg_utilities_1.lgUtils.glueFractionMembers(decl.value);
        declArr = sanitizedDecl.split(' ');
        const lostWaffle = declArr[0];
        if (declArr[1] !== undefined && declArr[1].search(/^\d/) !== -1) {
            lostWaffleCycle = declArr[1];
        }
        if (declArr[1] === 'flex' ||
            declArr[1] === 'no-flex' ||
            declArr[1] === 'auto') {
            lostWaffleCycle = declArr[0].split('/')[1];
        }
        if (declArr[2] !== undefined && declArr[2].search(/^\d/) !== -1) {
            lostWaffleGutter = declArr[2];
        }
        if (declArr.indexOf('flex') !== -1) {
            lostWaffleFlexbox = 'flex';
        }
        if (declArr.indexOf('no-flex') !== -1) {
            lostWaffleFlexbox = 'no-flex';
        }
        if (declArr.indexOf('float-right') !== -1) {
            floatRight = true;
        }
        decl.parent.nodes.forEach(function lostWaffleRounderFunction(declaration) {
            if (declaration.prop === 'lost-waffle-rounder') {
                lostWaffleRounder = declaration.value;
                declaration.remove();
            }
        });
        decl.parent.nodes.forEach(function lostWaffleCycleFunction(declaration) {
            if (declaration.prop === 'lost-waffle-cycle') {
                lostWaffleCycle = declaration.value;
                declaration.remove();
            }
        });
        // Converts the cycle to an integer so that checks on whether it's 0 make sense
        lostWaffleCycle = parseInt(lostWaffleCycle);
        decl.parent.nodes.forEach(function lostWaffleGutterFunction(declaration) {
            if (declaration.prop === 'lost-waffle-gutter') {
                lostWaffleGutter = declaration.value;
                declaration.remove();
            }
        });
        decl.parent.nodes.forEach((declaration) => {
            if (declaration.prop === 'lost-unit') {
                if (lg_logic_1.lgLogic.validateUnit(declaration.value, validUnits)) {
                    unit = declaration.value;
                }
                else {
                    throw declaration.error(`lost-unit: property ${declaration.value} is not a valid unit for lost-waffle.`);
                }
                declaration.remove();
            }
        });
        decl.parent.nodes.forEach(function lostWaffleFlexboxFunction(declaration) {
            if (declaration.prop === 'lost-waffle-flexbox') {
                if (declaration.value === 'flex') {
                    lostWaffleFlexbox = 'flex';
                }
                else {
                    throw declaration.error(`lost-waffle-flexbox: property '${declaration.value}' is unknown.`);
                }
                declaration.remove();
            }
        });
        if (lostWaffleFlexbox === 'flex') {
            decl.cloneBefore({
                prop: 'flex-grow',
                value: '0',
            });
            decl.cloneBefore({
                prop: 'flex-shrink',
                value: '0',
            });
            if (lostWaffleCycle !== 0) {
                (0, lg_new_block_1.newBlock)(decl, ':nth-last-child(-n + ' + lostWaffleCycle + ')', ['margin-bottom'], [0]);
                if (gridDirection === 'rtl') {
                    (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-left', 'margin-right'], [0, 'auto']);
                }
                else {
                    (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-right', 'margin-left'], [0, 'auto']);
                }
            }
            decl.cloneBefore({
                prop: 'flex-basis',
                value: lg_logic_1.lgLogic.calcValue(lostWaffle, lostWaffleGutter, lostWaffleRounder),
            });
            if (gridDirection === 'rtl') {
                (0, lg_new_block_1.newBlock)(decl, ':last-child', ['margin-left', 'margin-bottom'], [0, 0]);
                (0, lg_new_block_1.newBlock)(decl, ':nth-child(1n)', ['margin-left', 'margin-bottom', 'margin-right'], [lostWaffleGutter, lostWaffleGutter, 0]);
            }
            else {
                (0, lg_new_block_1.newBlock)(decl, ':last-child', ['margin-right', 'margin-bottom'], [0, 0]);
                (0, lg_new_block_1.newBlock)(decl, ':nth-child(1n)', ['margin-right', 'margin-bottom', 'margin-left'], [lostWaffleGutter, lostWaffleGutter, 0]);
            }
        }
        else {
            if (lostWaffleCycle !== 0) {
                (0, lg_new_block_1.newBlock)(decl, ':nth-last-child(-n + ' + lostWaffleCycle + ')', ['margin-bottom'], [0]);
                if (settings.clearing === 'left') {
                    // FIXME: this doesn't make sense w/ rtl
                    /* c8 ignore start */
                    if (gridDirection === 'rtl') {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n + 1)', ['clear'], ['right']);
                    }
                    else {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n + 1)', ['clear'], ['left']);
                    }
                }
                else {
                    (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n + 1)', ['clear'], ['both']);
                }
                /* c8 ignore end */
                if (gridDirection === 'rtl') {
                    // FIXME: this doesn't make sense w/ rtl
                    if (floatRight === true) {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-left', 'float'], [0, 'left']);
                    }
                    else {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-left'], [0]);
                    }
                }
                else {
                    if (floatRight === true) {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-right', 'float'], [0, 'right']);
                    }
                    else {
                        (0, lg_new_block_1.newBlock)(decl, ':nth-child(' + lostWaffleCycle + 'n)', ['margin-right'], [0]);
                    }
                }
            }
            if (gridDirection === 'rtl') {
                (0, lg_new_block_1.newBlock)(decl, ':last-child', ['margin-left', 'margin-bottom'], [0, 0]);
                (0, lg_new_block_1.newBlock)(decl, ':nth-child(1n)', ['float', 'margin-left', 'margin-bottom', 'clear'], ['right', lostWaffleGutter, lostWaffleGutter, 'none']);
            }
            else {
                (0, lg_new_block_1.newBlock)(decl, ':last-child', ['margin-right', 'margin-bottom'], [0, 0]);
                (0, lg_new_block_1.newBlock)(decl, ':nth-child(1n)', ['float', 'margin-right', 'margin-bottom', 'clear'], ['left', lostWaffleGutter, lostWaffleGutter, 'none']);
            }
        }
        cloneAllBefore({
            width: lg_logic_1.lgLogic.calcValue(lostWaffle, lostWaffleGutter, lostWaffleRounder, unit),
            'max-width': lg_logic_1.lgLogic.calcValue(lostWaffle, lostWaffleGutter, lostWaffleRounder, unit),
            height: lg_logic_1.lgLogic.calcValue(lostWaffle, lostWaffleGutter, lostWaffleRounder, unit),
        });
        decl.remove();
    });
};
exports.lostWaffle = lostWaffle;
